==> spirit_hub-0.1.1.6.tar.gz: spirit_hub-0.1.1.6/spec/services/spirit_hub/hubdoc_cycle_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::HubdocCycle, type: :service) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'default_cycle' do
    it 'returns the default cycle' do
      expect(described_class.default_cycle).to be_a(Cycle)
    end
  end

  describe 'possible_current_states' do
    it 'returns the possible current states' do
      expect(described_class.possible_current_states.length).to eq(16)
    end
  end

  describe 'import_cycle_from_json' do
    shared_examples 'does not create duplicates' do |model|
      it "does not create duplicate records for #{model}" do
        described_class.import_cycle_from_json

        expect { described_class.import_cycle_from_json }.not_to(change { model.count })
      end
    end

    shared_examples 'does not update without changes' do |model|
      it "does not update records for #{model} if no changes" do
        described_class.import_cycle_from_json

        Timecop.freeze(10.day.from_now) do
          expect { described_class.import_cycle_from_json }.not_to(change { model.pluck(:updated_at).map(&:to_date) })
        end
      end
    end

    it_behaves_like 'does not create duplicates', Cycle
    it_behaves_like 'does not create duplicates', State
    it_behaves_like 'does not create duplicates', Trigger
    it_behaves_like 'does not create duplicates', Transition

    it_behaves_like 'does not update without changes', Cycle
    it_behaves_like 'does not update without changes', State
    it_behaves_like 'does not update without changes', Trigger
    it_behaves_like 'does not update without changes', Transition

    it 'creates initial records' do
      cycle_id = Cycle.first.id
      expect(Cycle.count).to eq(1)
      expect(State.where(cycle_id:).count).to eq(14)
      expect(Trigger.where(cycle_id:).count).to eq(16)
      expect(Transition.where(cycle_id:).count).to eq(14)
      expect(Slink.where(cycle_id:).count).to eq(2)
    end

    it 'delete state cycle not in json' do
      create(:state, cycle_id: Cycle.first.id, key: 'new_state')
      expect { described_class.import_cycle_from_json }.to change { State.where(cycle_id: Cycle.first.id).count }.by(-1)
    end
  end

  describe 'all_next_possible_states' do
    let!(:current_state) { nil }
    let!(:default_states) { %w[classify done generate_linked_document respond share sign to_answer to_complete to_share to_sign to_validate] }

    subject { described_class.all_next_possible_states(current_state).pluck(:key) }

    it 'when nil' do
      expect(subject).to match_array(default_states)
    end

    context 'with to_validate' do
      let!(:current_state) { described_class.find_state_by_key(:to_validate) }

      it 'returns validate/refuse' do
        expect(subject).to match_array(%w[validate refuse] + default_states - %w[to_validate])
      end
    end

    shared_examples 'return the default states' do |state_key|
      context "with #{state_key}" do
        let!(:current_state) { described_class.find_state_by_key(state_key) }

        it 'works' do
          expect(subject).to match_array(default_states - [state_key])
        end
      end
    end

    %w[classify done generate_linked_document respond share sign to_answer to_classify to_complete to_share to_sign].each do |state_key|
      it_behaves_like 'return the default states', state_key
    end
  end

  describe 'next_states' do
    let!(:current_state) { nil }

    subject { described_class.next_states(current_state).pluck(:key) }

    it 'when nil' do
      expect(subject).to be_empty
    end

    context 'with to_validate' do
      let!(:current_state) { described_class.find_state_by_key(:to_validate) }

      it 'returns validate/refuse' do
        expect(subject).to match_array(%w[validate refuse])
      end
    end

    shared_examples 'return empty next_state' do |state_key|
      context "with #{state_key}" do
        let!(:current_state) { described_class.find_state_by_key(state_key) }

        it 'works' do
          expect(subject).to be_empty
        end
      end
    end

    %w[classify done generate_linked_document respond share sign to_answer to_classify to_complete to_share to_sign].each do |state_key|
      it_behaves_like 'return empty next_state', state_key
    end
  end

  describe 'can_change_to_state?' do
    let!(:admin) { create(:user, roles: ['Administrator']) }
    let!(:responsible) { create(:user) }
    let!(:reader) { create(:user) }
    let!(:writer) { create(:user) }
    let!(:accounted) { create(:user) }
    let!(:other_user) { create(:user) }
    let!(:tagset) do
      create(
        :tagset_doc,
        responsible: responsible.id,
        readers: [reader.id.to_s],
        writers: [writer.id.to_s],
        accounted: [accounted.id.to_s]
      )
    end
    let(:state_key) { :sent_to_validate }

    subject { described_class.can_change_to_state?(tagset, state_key, user) }

    [
      [:admin, true],
      [:responsible, true],
      [:reader, false],
      [:writer, true],
      [:accounted, true],
      [:other_user, false]
    ].each do |user_name, expected_result|
      context "for #{user_name}" do
        let(:user) { send(user_name) }

        it "returns #{expected_result}" do
          expect(subject).to eq(expected_result)
        end
      end
    end

    context 'share action' do
      let(:state_key) { :share }
      let(:user) { other_user }

      it 'returns true' do
        expect(subject).to eq(true)
      end
    end

    context 'invalid model' do
      let(:user) { other_user }
      let(:tagset) { create(:doc) }

      it 'returns false' do
        expect(subject).to eq(false)
      end
    end
  end
end

==> spirit_hub-0.1.1.22.tar.gz: spirit_hub-0.1.1.22/spec/services/spirit_hub/hubdoc_cycle_spec.rb <==
# frozen_string_literal: true

require 'rails_helper'

RSpec.describe(SpiritHub::HubdocCycle, type: :service) do
  before do
    SpiritHub::HubdocCycle.import_cycle_from_json
  end

  describe 'default_cycle' do
    it 'returns the default cycle' do
      expect(described_class.default_cycle).to be_a(Cycle)
    end
  end

  describe 'possible_current_states' do
    it 'returns the possible current states' do
      expect(described_class.possible_current_states.length).to eq(15)
    end
  end

  describe 'import_cycle_from_json' do
    shared_examples 'does not create duplicates' do |model|
      it "does not create duplicate records for #{model}" do
        described_class.import_cycle_from_json

        expect { described_class.import_cycle_from_json }.not_to(change { model.count })
      end
    end

    shared_examples 'does not update without changes' do |model|
      it "does not update records for #{model} if no changes" do
        described_class.import_cycle_from_json

        Timecop.freeze(10.day.from_now) do
          expect { described_class.import_cycle_from_json }.not_to(change { model.pluck(:updated_at).map(&:to_date) })
        end
      end
    end

    it_behaves_like 'does not create duplicates', Cycle
    it_behaves_like 'does not create duplicates', State
    it_behaves_like 'does not create duplicates', Trigger
    it_behaves_like 'does not create duplicates', Transition

    it_behaves_like 'does not update without changes', Cycle
    it_behaves_like 'does not update without changes', State
    it_behaves_like 'does not update without changes', Trigger
    it_behaves_like 'does not update without changes', Transition

    it 'creates initial records' do
      cycle_id = Cycle.first.id
      expect(Cycle.count).to eq(1)
      expect(State.where(cycle_id:).count).to eq(13)
      expect(Trigger.where(cycle_id:).count).to eq(16)
      expect(Transition.where(cycle_id:).count).to eq(13)
      expect(Slink.where(cycle_id:).count).to eq(2)
    end

    it 'delete state cycle not in json' do
      create(:state, cycle_id: Cycle.first.id, key: 'new_state')
      expect { described_class.import_cycle_from_json }.to change { State.where(cycle_id: Cycle.first.id).count }.by(-1)
    end
  end

  describe 'all_next_possible_states' do
    let!(:current_state) { nil }
    let!(:default_states) { %w[classify done generate_linked_document respond share to_answer to_complete to_share to_sign to_validate] }

    subject { described_class.all_next_possible_states(current_state).pluck(:key) }

    it 'when nil' do
      expect(subject).to match_array(default_states)
    end

    context 'with to_validate' do
      let!(:current_state) { described_class.find_state_by_key(:to_validate) }

      it 'returns validate/refuse' do
        expect(subject).to match_array(%w[validate refuse] + default_states - %w[to_validate])
      end
    end

    shared_examples 'return the default states' do |state_key|
      context "with #{state_key}" do
        let!(:current_state) { described_class.find_state_by_key(state_key) }

        it 'works' do
          expect(subject).to match_array(default_states - [state_key])
        end
      end
    end

    %w[classify done generate_linked_document respond share to_answer to_classify to_complete to_share to_sign].each do |state_key|
      it_behaves_like 'return the default states', state_key
    end
  end

  describe 'next_states' do
    let!(:current_state) { nil }

    subject { described_class.next_states(current_state).pluck(:key) }

    it 'when nil' do
      expect(subject).to be_empty
    end

    context 'with to_validate' do
      let!(:current_state) { described_class.find_state_by_key(:to_validate) }

      it 'returns validate/refuse' do
        expect(subject).to match_array(%w[validate refuse])
      end
    end

    shared_examples 'return empty next_state' do |state_key|
      context "with #{state_key}" do
        let!(:current_state) { described_class.find_state_by_key(state_key) }

        it 'works' do
          expect(subject).to be_empty
        end
      end
    end

    %w[classify done generate_linked_document respond share to_answer to_classify to_complete to_share to_sign].each do |state_key|
      it_behaves_like 'return empty next_state', state_key
    end
  end

  describe 'can_change_to_state?' do
    let!(:admin) { create(:user, roles: ['Administrator']) }
    let!(:responsible) { create(:user) }
    let!(:reader) { create(:user) }
    let!(:writer) { create(:user) }
    let!(:accounted) { create(:user) }
    let!(:other_user) { create(:user) }
    let!(:tagset) do
      create(
        :tagset_doc,
        responsible: responsible.id,
        readers: [reader.id.to_s],
        writers: [writer.id.to_s],
        accounted: [accounted.id.to_s]
      )
    end
    let(:state_key) { :sent_to_validate }

    subject { described_class.can_change_to_state?(tagset, state_key, user) }

    [
      [:admin, true],
      [:responsible, true],
      [:reader, false],
      [:writer, true],
      [:accounted, true],
      [:other_user, false]
    ].each do |user_name, expected_result|
      context "for #{user_name}" do
        let(:user) { send(user_name) }

        it "returns #{expected_result}" do
          expect(subject).to eq(expected_result)
        end
      end
    end

    context 'share action' do
      let(:state_key) { :share }
      let(:user) { other_user }

      it 'returns true' do
        expect(subject).to eq(true)
      end
    end

    context 'invalid model' do
      let(:user) { other_user }
      let(:tagset) { create(:doc) }

      it 'returns false' do
        expect(subject).to eq(false)
      end
    end
  end
end
